@@ -1,6 +1,10 @@
 #include "esd-server.h"
 
 #include <arpa/inet.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <sys/un.h>
+#include <errno.h>
 
 #ifndef HAVE_NANOSLEEP
 #include <sys/time.h>
@@ -40,7 +44,9 @@ int esd_autostandby_secs = -1; 	/* timeout to release audio device, disabled <0
 time_t esd_last_activity = 0;	/* seconds since last activity */
 int esd_on_autostandby = 0;	/* set when auto paused for auto reawaken */
 
-
+int esd_use_tcpip = 0;          /* use tcp/ip sockets instead of unix domain */
+int esd_terminate = 0;          /* terminate after the last client exits */
+int esd_public = 0;             /* allow connects from hosts other than localhost */
 /*******************************************************************/
 /* just to create the startup tones for the fun of it */
 void set_audio_buffer( void *buf, esd_format_t format,
@@ -130,7 +136,7 @@ void clean_exit(int signum) {
     /* just clean up as best we can and terminate from here */
     esd_client_t * client = esd_clients_list;
     
-    fprintf( stderr, "received signal %d: terminating...\n", signum );
+/*    fprintf( stderr, "received signal %d: terminating...\n", signum );*/
     
     /* free the sound device */
     esd_audio_close();
@@ -144,19 +150,51 @@ void clean_exit(int signum) {
 	close( client->fd );
 	client = client->next;
     }
+   if (!esd_use_tcpip)
+    {
+      unlink(ESD_UNIX_SOCKET_NAME);
+      unlink(ESD_UNIX_SOCKET_DIR);
+    }
+  
 
     /* trust the os to clean up the memory for the samples and such */
-    fprintf( stderr, "bye bye.\n" );
     exit( 0 );
 }
 
 void reset_signal(int signum) {
-    fprintf( stderr, "received signal %d: resetting...\n", signum );
+/*    fprintf( stderr, "received signal %d: resetting...\n", signum );*/
     signal( signum, reset_signal);
 
     return;
 }
 
+static int
+esd_connect_unix(void)
+{
+  struct sockaddr_un socket_unix;
+  int socket_out = -1;
+  int curstate = 1;
+  
+  /* create the socket, and set for non-blocking */
+  socket_out = socket( AF_UNIX, SOCK_STREAM, 0 );
+  if ( socket_out < 0 )
+    return -1;
+  /* this was borrowed blindly from the Tcl socket stuff */
+  if ( fcntl( socket_out, F_SETFD, FD_CLOEXEC ) < 0 )
+    return -1;
+  if ( setsockopt( socket_out, SOL_SOCKET, SO_REUSEADDR,
+		  &curstate, sizeof(curstate) ) < 0 )
+    return -1;  
+  /* set the connect information */
+  socket_unix.sun_family = AF_UNIX;
+  strncpy(socket_unix.sun_path, ESD_UNIX_SOCKET_NAME, sizeof(socket_unix.sun_path));  
+  if ( connect( socket_out,
+	       (struct sockaddr *) &socket_unix, SUN_LEN(&socket_unix) ) < 0 )
+    return -1;  
+  return socket_out;
+}
+
+
 /*******************************************************************/
 /* returns the listening socket descriptor */
 int open_listen_socket( int port )
@@ -164,11 +202,56 @@ int open_listen_socket( int port )
     /*********************/
     /* socket test setup */
     struct sockaddr_in socket_addr;
-    int socket_listen;
+    struct sockaddr_un socket_unix;
+    int socket_listen = -1;
     struct linger lin;
+      
    
     /* create the socket, and set for non-blocking */
-    socket_listen=socket(AF_INET,SOCK_STREAM,0);
+    if (esd_use_tcpip)
+      socket_listen=socket(AF_INET,SOCK_STREAM,0);
+    else
+    {
+      if (access(ESD_UNIX_SOCKET_DIR, R_OK | W_OK) == -1)
+	{	
+	  mkdir(ESD_UNIX_SOCKET_DIR,
+		S_IRUSR|S_IWUSR|S_IXUSR|
+		S_IRGRP|S_IWGRP|S_IXGRP|
+		S_IROTH|S_IWOTH|S_IXOTH);
+	  chmod(ESD_UNIX_SOCKET_DIR,
+		S_IRUSR|S_IWUSR|S_IXUSR|
+		S_IRGRP|S_IWGRP|S_IXGRP|
+		S_IROTH|S_IWOTH|S_IXOTH);
+	}
+      if (access(ESD_UNIX_SOCKET_NAME, R_OK | W_OK) == -1)
+	{
+	  if (errno == EACCES)
+	    {
+	      /* not allowed access */
+	      fprintf(stderr, 
+		      "esd: Esound sound daemon unable to create unix domain socket:\n"
+		      "%s\n"
+		      "This socket already exists and is not accessible by esd.\n"
+		      "Exiting...\n", ESD_UNIX_SOCKET_NAME);
+	      exit(1);
+	    }
+	}
+      else
+	{
+	  if (esd_connect_unix() >= 0)
+	    {
+	      /* not allowed access */
+	      fprintf(stderr, 
+		      "esd: Esound sound daemon already running or stale UNIX socket\n"
+		      "%s\n"
+		      "This socket already exists indicating esd is already running.\n"
+		      "Exiting...\n", ESD_UNIX_SOCKET_NAME);
+	      exit(1);
+	    }
+	}
+      unlink(ESD_UNIX_SOCKET_NAME);
+      socket_listen=socket(AF_UNIX,SOCK_STREAM,0);
+    }
     if (socket_listen<0) 
     {
 	fprintf(stderr,"Unable to create socket\n");
@@ -196,21 +279,54 @@ int open_listen_socket( int port )
       /* if it fails, so what */
     }
 
-    /* set the listening information */
-    socket_addr.sin_family = AF_INET;
-    socket_addr.sin_port = htons( port );
-    socket_addr.sin_addr.s_addr = htonl( inet_addr("0.0.0.0") );
-    if ( bind( socket_listen,
-	       (struct sockaddr *) &socket_addr,
-	       sizeof(struct sockaddr_in) ) < 0 )
+    if (esd_use_tcpip)
     {
-	fprintf(stderr,"Unable to bind port %d\n", port );
-	exit(1);
+      /* set the listening information */
+      socket_addr.sin_family = AF_INET;
+      socket_addr.sin_port = htons( port );
+      if (esd_public)
+	socket_addr.sin_addr.s_addr = htonl( INADDR_ANY );
+      else
+	socket_addr.sin_addr.s_addr = htonl( INADDR_LOOPBACK );
+      if ( bind( socket_listen,
+		(struct sockaddr *) &socket_addr,
+		sizeof(struct sockaddr_in) ) < 0 )
+	{
+	  fprintf(stderr,"Unable to bind port %d\n", port );
+	  exit(1);
+	}
+    }
+    else
+    {
+      socket_unix.sun_family=AF_UNIX;
+      strncpy(socket_unix.sun_path, ESD_UNIX_SOCKET_NAME, sizeof(socket_unix.sun_path));
+      if ( bind( socket_listen,
+		(struct sockaddr *) &socket_unix, SUN_LEN(&socket_unix) ) < 0 )
+	{
+	  fprintf(stderr,"Unable to connect to UNIX socket %s\n", ESD_UNIX_SOCKET_NAME);
+	  if (!esd_use_tcpip)
+	    {
+	      unlink(ESD_UNIX_SOCKET_NAME);
+	      unlink(ESD_UNIX_SOCKET_DIR);
+	    }
+	  exit(1);
+	}
+      /* let anyone access esd's socket - but we have authentication so they */
+      /* wont get far if they dont have the auth key */
+      chmod(ESD_UNIX_SOCKET_NAME, 
+	    S_IRUSR|S_IWUSR|S_IXUSR|
+	    S_IRGRP|S_IWGRP|S_IXGRP|
+	    S_IROTH|S_IWOTH|S_IXOTH);
     }
     if (listen(socket_listen,16)<0)
     {
-	fprintf(stderr,"Unable to set socket listen buffer length\n");
-	exit(1);
+      fprintf(stderr,"Unable to set socket listen buffer length\n");
+      if (!esd_use_tcpip)
+	{
+	  unlink(ESD_UNIX_SOCKET_NAME);
+	  unlink(ESD_UNIX_SOCKET_DIR);
+	}
+      exit(1);
     }
 
     return socket_listen;
@@ -268,6 +384,7 @@ int main ( int argc, char *argv[] )
     int esd_port = ESD_DEFAULT_PORT;
     int length = 0;
     int arg = 0;
+    int itmp;
 
     void *output_buffer = NULL;
 
@@ -278,6 +395,8 @@ int main ( int argc, char *argv[] )
     int i, j, freq=440;
     int magl, magr;
 
+    int first = 1;
+
     int default_format = ESD_BITS16 | ESD_STEREO;
     /* end test scaffolding parameters */
 
@@ -327,8 +446,8 @@ int main ( int argc, char *argv[] )
 		    fprintf( stderr, "- could not read autostandby timeout: %s\n", 
 			     argv[ arg ] );
 		}
-		fprintf( stderr, "- autostandby timeout: %d seconds\n", 
-			 esd_autostandby_secs );
+/*		fprintf( stderr, "- autostandby timeout: %d seconds\n", 
+			 esd_autostandby_secs );*/
 	    }
 #ifdef ESDBG
 	} else if ( !strcmp( argv[ arg ], "-vt" ) ) {
@@ -343,19 +462,31 @@ int main ( int argc, char *argv[] )
 #endif
 	} else if ( !strcmp( argv[ arg ], "-nobeeps" ) ) {
 	    esd_beeps = 0;
-	    fprintf( stderr, "- disabling startup beeps\n" );
+/*	    fprintf( stderr, "- disabling startup beeps\n" );*/
+	} else if ( !strcmp( argv[ arg ], "-unix" ) ) {
+	    esd_use_tcpip = 0;
+	} else if ( !strcmp( argv[ arg ], "-tcp" ) ) {
+	    esd_use_tcpip = 1;
+	} else if ( !strcmp( argv[ arg ], "-public" ) ) {
+	    esd_public = 1;
+	} else if ( !strcmp( argv[ arg ], "-terminate" ) ) {
+	    esd_terminate = 1;
 	} else if ( !strcmp( argv[ arg ], "-h" ) ) {
 	    fprintf( stderr, "Usage: esd [options]\n\n" );
 	    fprintf( stderr, "  -d DEVICE   force esd to use sound device DEVICE\n" );
 	    fprintf( stderr, "  -b          run server in 8 bit sound mode\n" );
 	    fprintf( stderr, "  -r RATE     run server at sample rate of RATE\n" );
 	    fprintf( stderr, "  -as SECS    free audio device after SECS of inactivity\n" );
+	    fprintf( stderr, "  -unix       use unix domain sockets instead of tcp/ip\n" );
+	    fprintf( stderr, "  -tcp        use tcp/ip sockets instead of unix domain\n" );
+	    fprintf( stderr, "  -public     make tcp/ip access public (other than localhost)\n" );
+	    fprintf( stderr, "  -terminate  terminate esd daemone after last client exits\n" );
 #ifdef ESDBG
 	    fprintf( stderr, "  -vt         enable trace diagnostic info\n" );
 	    fprintf( stderr, "  -vc         enable comms diagnostic info\n" );
 	    fprintf( stderr, "  -vm         enable mixer diagnostic info\n" );
 #endif
-	    fprintf( stderr, "  -port PORT  listen for connections at PORT\n" );
+	    fprintf( stderr, "  -port PORT  listen for connections at PORT (only for tcp/ip)\n" );
 	    fprintf( stderr, "\nPossible devices are:  %s\n", esd_audio_devices() );
 	    exit( 0 );
 	} else {
@@ -363,35 +494,115 @@ int main ( int argc, char *argv[] )
 	}
     }
 
+    /* open the listening socket */
+    listen_socket = open_listen_socket( esd_port );
+  if ( listen_socket < 0 ) {
+    fprintf( stderr, "fatal error opening socket\n" );
+    if (!esd_use_tcpip)
+      {
+	unlink(ESD_UNIX_SOCKET_NAME);
+	unlink(ESD_UNIX_SOCKET_DIR);
+      }
+    exit( 1 );	    
+  }
+  
+#define ESD_AUDIO_STUFF \
+    esd_sample_size = ( (esd_audio_format & ESD_MASK_BITS) == ESD_BITS16 ) \
+	? sizeof(signed short) : sizeof(unsigned char); \
+    esd_buf_size_samples = default_buf_size / 2; \
+    esd_buf_size_octets = esd_buf_size_samples * esd_sample_size;
+
     /* start the initializatin process */
-    printf( "initializing...\n" );
+/*    printf( "ESound ESD daemon initializing...\n" );*/
 
     /* set the data size parameters */
     esd_audio_format = default_format;
     esd_audio_rate = default_rate;
+    ESD_AUDIO_STUFF;
 
-    esd_sample_size = ( (esd_audio_format & ESD_MASK_BITS) == ESD_BITS16 )
-	? sizeof(signed short) : sizeof(unsigned char);
-    esd_buf_size_samples = default_buf_size / 2;
-    esd_buf_size_octets = esd_buf_size_samples * esd_sample_size;
-
-    /* open and initialize the audio device, /dev/dsp */
+  /* open and initialize the audio device, /dev/dsp */
+  itmp = esd_audio_open();
+  if (itmp == -2) { /* Special return value indicates open of device failed, don't bother
+		       trying */
+	exit (2);
+  } else if ( itmp < 0 ) {
+    fprintf(stderr, "Audio device open for 44.1Khz, stereo, 16bit failed\n"
+	    "Trying 44.1Khz, 8bit stereo.\n");
+    /* cant do defaults ... try 44.1 kkz 8bit stereo */
+    esd_audio_format = ESD_BITS8 | ESD_STEREO;
+    esd_audio_rate = 44100;
+    ESD_AUDIO_STUFF;
     if ( esd_audio_open() < 0 ) {
-	fprintf( stderr, "fatal error configuring sound, %s\n", 
-		 "/dev/dsp" );
-	exit( 1 );	    
+      fprintf(stderr, "Audio device open for 44.1Khz, stereo, 8bit failed\n"
+	      "Trying 22.05Khz, 8bit stereo.\n");
+      /* cant do defaults ... try 22.05 kkz 8bit stereo */
+      esd_audio_format = ESD_BITS8 | ESD_STEREO;
+      esd_audio_rate = 22050;
+      ESD_AUDIO_STUFF;
+      if ( esd_audio_open() < 0 ) {
+	fprintf(stderr, "Audio device open for 22.05Khz, stereo, 8bit failed\n"
+		"Trying 44.1Khz, 16bit mono.\n");
+	/* cant do defaults ... try 44.1Khz kkz 16bit mono */
+	esd_audio_format = ESD_BITS16;
+	esd_audio_rate = 44100;
+	ESD_AUDIO_STUFF;
+	if ( esd_audio_open() < 0 ) {
+	  fprintf(stderr, "Audio device open for 44.1Khz, mono, 8bit failed\n"
+		  "Trying 22.05Khz, 8bit mono.\n");
+	  /* cant do defaults ... try 22.05 kkz 8bit mono */
+	  esd_audio_format = ESD_BITS8;
+	  esd_audio_rate = 22050;
+	  ESD_AUDIO_STUFF;
+	  if ( esd_audio_open() < 0 ) {
+	    fprintf(stderr, "Audio device open for 22.05Khz, mono, 8bit failed\n"
+		    "Trying 11.025Khz, 8bit stereo.\n");
+	    /* cant to defaults ... try 11.025 kkz 8bit stereo */
+	    esd_audio_format = ESD_BITS8 | ESD_STEREO;
+	    esd_audio_rate = 11025;
+	    ESD_AUDIO_STUFF;
+	    if ( esd_audio_open() < 0 ) {
+	      fprintf(stderr, "Audio device open for 11.025Khz, stereo, 8bit failed\n"
+		      "Trying 11.025Khz, 8bit mono.\n");
+	      /* cant to defaults ... try 11.025 kkz 8bit mono */
+	      esd_audio_format = ESD_BITS8;
+	      esd_audio_rate = 11025;
+	      ESD_AUDIO_STUFF;
+	      if ( esd_audio_open() < 0 ) {
+		fprintf(stderr, "Audio device open for 11.025Khz, mono, 8bit failed\n"
+			"Trying 8.192Khz, 8bit mono.\n");
+	        /* cant to defaults ... try 8.192 kkz 8bit mono */
+		esd_audio_format = ESD_BITS8;
+		esd_audio_rate = 8192;
+		ESD_AUDIO_STUFF;
+		if ( esd_audio_open() < 0 ) {
+		  fprintf(stderr, "Audio device open for 8.192Khz, mono, 8bit failed\n"
+			  "Trying 8Khz, 8bit mono.\n");
+		  /* cant to defaults ... try 8 kkz 8bit mono */
+		  esd_audio_format = ESD_BITS8;
+		  esd_audio_rate = 8000;
+		  ESD_AUDIO_STUFF;
+		  if ( esd_audio_open() < 0 ) {
+		    fprintf(stderr, "Sound device inadequate for Esound. Fatal.\n");
+		    if (!esd_use_tcpip)
+		      {
+			unlink(ESD_UNIX_SOCKET_NAME);
+			unlink(ESD_UNIX_SOCKET_DIR);
+		      }
+		    exit( 1 );
+		  }
+		}
+	      }
+	    }
+	  }
+	}
+      }
     }
+  }
 
     /* allocate and zero out buffer */
     output_buffer = (void *) malloc( esd_buf_size_octets );
     memset( output_buffer, 0, esd_buf_size_octets );
 
-    /* open the listening socket */
-    listen_socket = open_listen_socket( esd_port );
-    if ( listen_socket < 0 ) {
-	fprintf( stderr, "fatal error opening socket\n" );
-	exit( 1 );	    
-    }
     
     /* install signal handlers for program integrity */
     signal( SIGINT, clean_exit );	/* for ^C */
@@ -429,8 +640,16 @@ int main ( int argc, char *argv[] )
 	/* accept new connections */
 	get_new_clients( listen_socket );
 
+      
+	if ((esd_clients_list == NULL) && (!first) && (esd_terminate)) {
+/*	  fprintf(stderr, "No clients!\n");*/
+	  clean_exit(0);
+	  exit(0);
+	}
+
 	/* check for new protocol requests */
 	poll_client_requests();
+	first = 0;
 
 	/* mix new requests, and output to device */
 	refresh_mix_funcs(); /* TODO: set a flag to cue when to do this */
